#ifndef _3DMATH_H_
#define _3DMATH_H_

#ifndef PI
#define PI 3.1415926535897932					// This is our famous PI
#endif


#define SPHERE_POS_BEHIND		0
#define SPHERE_POS_INTERSECTS	1
#define SPHERE_POS_FRONT		2

// This is used to hold 3D points and vectors.  This will eventually be added upon
// to create a robut class with operator overloading.  For now we just need an x, y, z.

// lze pouzit jako barva i jako souradnice
/*
struct HVector3
{
	union {
	// 3D souradnice
		struct { float x, y, z; };
	// 3D barva
		struct { float r, g, b; };
	};
};
*/
/*
union HVector3{
// 3D souradnice
	struct { float x, y, z; };
// 3D barva
	struct { float r, g, b; };
};
*/


// This is our 3D point class.  This will be used to store the vertices of our model.
class CVector3
{
public:
	float x, y, z;
};

// This is our 2D point class.  This will be used to store the UV coordinates.
class CVector2
{
public:
	float x, y;
};

// operatory
// porovnani
bool operator == (CVector3 v1, CVector3 v2);
// vynasobeni cislem
CVector3 operator * (CVector3 v, float f);
CVector3& operator *= (CVector3 &v, float f);

// odecteni c = a - b;  - odectou se souradnice
CVector3 operator - (CVector3 a, CVector3  b);

// odecteni c = a + b;  - sectou se souradnice
CVector3 operator + (CVector3 a, CVector3  b);


// nastavi vektor na 0, 0, 0
void ZeroVector(CVector3 *pvector);


// vypocita vektor podle dvou bodu
CVector3 Vector(CVector3 vPoint1, CVector3 vPoint2);


// This adds 2 vectors together and returns the result
CVector3 AddVector(CVector3 vVector1, CVector3 vVector2);


// This divides a vector by a single number (scalar) and returns the result
CVector3 DivideVectorByScaler(CVector3 vVector1, float Scaler);


// This returns the cross product between 2 vectors
CVector3 Cross(CVector3 vVector1, CVector3 vVector2);


// This returns the normal of a vector
CVector3 Normalize(CVector3 vNormal);

// This returns the absolute value of "num"
float Absolute(float num);

//	This returns a perpendicular vector from 2 given vectors by taking the cross product.
CVector3 Cross(CVector3 vVector1, CVector3 vVector2);

//	This returns the magnitude of a normal (or any other vector)
float Magnitude(CVector3 vNormal);

//	This returns a normalize vector (A vector exactly of length 1)
CVector3 Normalize(CVector3 vNormal);

//	This returns the normal of a polygon (The direction the polygon is facing)
CVector3 Normal(CVector3 vPolygon[]);

// This returns the distance between 2 3D points
float Distance(CVector3 vPoint1, CVector3 vPoint2);

// This returns the point on the line segment vA_vB that is closest to point vPoint
CVector3 ClosestPointOnLine(CVector3 vA, CVector3 vB, CVector3 vPoint);

// This returns the distance the plane is from the origin (0, 0, 0)
// It takes the normal to the plane, along with ANY point that lies on the plane (any corner)
float PlaneDistance(CVector3 Normal, CVector3 Point);

// This takes a triangle (plane) and line and returns true if they intersected
bool IntersectedPlane(CVector3 vPoly[], CVector3 vLine[], CVector3 &vNormal, float &originDistance);

// This returns the dot product between 2 vectors
float Dot(CVector3 vVector1, CVector3 vVector2);

// This returns the angle between 2 vectors
double AngleBetweenVectors(CVector3 Vector1, CVector3 Vector2);

// This returns an intersection point of a polygon and a line (assuming intersects the plane)
CVector3 IntersectionPoint(CVector3 vNormal, CVector3 vLine[], double distance);

// This returns true if the intersection point is inside of the polygon
bool InsidePolygon(CVector3 vIntersection, CVector3 Poly[], long verticeCount);

// Use this function to test collision between a line and polygon
bool IntersectedPolygon(CVector3 vPoly[], CVector3 vLine[], int verticeCount);

// This function classifies a sphere according to a plane. (BEHIND, in FRONT, or INTERSECTS)
int ClassifySphere(CVector3 &vCenter, 
				   CVector3 &vNormal, CVector3 &vPoint, float radius, float &distance);

// This takes in the sphere center, radius, polygon vertices and vertex count.
// This function is only called if the intersection point failed.  The sphere
// could still possibly be intersecting the polygon, but on it's edges.
bool EdgeSphereCollision(CVector3 &vCenter, 
						 CVector3 vPolygon[], int vertexCount, float radius);

// This returns true if the sphere is intersecting with the polygon.
bool SpherePolygonCollision(CVector3 vPolygon[], 
							CVector3 &vCenter, int vertexCount, float radius);



/*
struct HVector3
{
public:
	float x, y, z;						
};


//	This returns a perpendicular vector from 2 given vectors by taking the cross product.
HVector3 Cross(HVector3 vVector1, HVector3 vVector2);

//	This returns a vector between 2 points
HVector3 Vector(HVector3 vPoint1, HVector3 vPoint2);

//	This returns the magnitude of a normal (or any other vector)
float Magnitude(HVector3 vNormal);

//	This returns a normalize vector (A vector exactly of length 1)
HVector3 Normalize(HVector3 vNormal);

//	This returns the normal of a polygon (The direction the polygon is facing)
HVector3 Normal(HVector3 vTriangle[]);

// This returns the distance the plane is from the origin (0, 0, 0)
// It takes the normal to the plane, along with ANY point that lies on the plane (any corner)
float PlaneDistance(HVector3 Normal, HVector3 Point);

// This takes a triangle (plane) and line and returns true if they intersected
bool IntersectedPlane(HVector3 vPoly[], HVector3 vLine[], HVector3 &vNormal, float &originDistance);

// This returns the dot product between 2 vectors
float Dot(HVector3 vVector1, HVector3 vVector2);

// This returns the angle between 2 vectors
double AngleBetweenVectors(HVector3 Vector1, HVector3 Vector2);

// This returns an intersection point of a polygon and a line (assuming intersects the plane)
HVector3 IntersectionPoint(HVector3 vNormal, HVector3 vLine[], double distance);

// This returns true if the intersection point is inside of the polygon
bool InsidePolygon(HVector3 vIntersection, HVector3 Poly[], long verticeCount);

// Use this function to test collision between a line and polygon
bool IntersectedPolygon(HVector3 vPoly[], HVector3 vLine[], int verticeCount);
*/

#endif 


/////////////////////////////////////////////////////////////////////////////////
//
// * QUICK NOTES * 
//
// We added 4 new math functions to our library.  A few of these functions will 
// be used constantly throughout your 3D programming careers, so make sure you know them.
// They are :  Dot()	AngleBetweenVectors()	IntersectionPoint()   InsidePolygon()
//
// We also added a helper function called IntersectedPolygon(), which calls
// IntersectionPoint() and InsidePolygon().  This is the function that the client
// will call.  We don't need to call the other 2 functions by themselves anywhere else.
//
// From now on, the dot product will be one of the biggest math functions you will use.
// It's importance is right behind vectors and normals.
//
// Read more about these functions in 3DMath.cpp
//
//
// Ben Humphrey (DigiBen)
// Game Programmer
// DigiBen@GameTutorials.com
// Co-Web Host of www.GameTutorials.com
//
//


