//ENGINE

// GActiveObject.cpp: implementation of the GActiveObject class.
//
//////////////////////////////////////////////////////////////////////

#include "ph.h"
#include "GActiveObject.h"
#include "GAI.h"
#include "H3DModel2.h"
#include "GEngine.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GActiveObject::GActiveObject(GEngine *engine, H3DModel *model3D):
GObject(engine)
{
	m_speed = 0;
	m_acceleration = 1;
	
	m_up = false;
	m_down = false;
	m_left = false;
	m_right = false;
	m_forward = false;  
	m_backward = false; 
	m_rotY = 0;     
	m_rotX = 0;     
	m_rotZ = 0;
	m_sin_alpha = 0;
	m_cos_alpha = 1;
	m_action1 = false;
	m_action2 = false;
	m_action3 = false;
	m_action4 = false;
	m_action5 = false;	
	
	m_3Dmodel = model3D;
	m_active = true;

	m_veight = 1;
	m_antislide = 0.0; // defaultne je slidovani vypnute
	
	m_ai = NULL;

	m_type = GOBJ_ACTIVE;

	// nulovani vektoru
	ZeroVector(&m_moveing);
	ZeroVector(&m_moveing);
	ZeroVector(&m_req_movement);
	ZeroVector(&m_rotation);
	ZeroVector(&m_sliding);
	ZeroVector(&m_model_position);
	
	// opiseme velikost objektu podle modelu
	m_radius = m_3Dmodel->m_cubeSide; // neni to sice uplne presne, ale co se da delat :-)
	m_size.x = (m_3Dmodel->m_boxMax.x - m_3Dmodel->m_boxMin.x)/2;
	m_size.y = (m_3Dmodel->m_boxMax.y - m_3Dmodel->m_boxMin.y)/2;
	m_size.z = (m_3Dmodel->m_boxMax.z - m_3Dmodel->m_boxMin.z)/2;
	// jeste by se mela m_size, ale vzhledem k otaceni objetu nam snad bude stacit jen tohle
}

/////////////////////////////////////////////////////////////////////////////////////////

GActiveObject::~GActiveObject()
{
	if (m_ai != NULL) delete m_ai;
}

/////////////////////////////////////////////////////////////////////////////////////////

void GActiveObject::DoEvent()
{
	// uz se stejne chceme smazat, takze to nebudeme hrotit :-)
	if (m_want_be_deleted) return;
	
	// dejme sanci AI
	if (m_ai != NULL) m_ai->Move();
		
	// aby nedochazelo k nejasnym situacim
	if (m_up && m_down) { m_up = false; m_down = false; }
	if (m_left  && m_right) { m_left = false; m_right = false ; }
	if (m_forward  && m_backward) { m_forward = false; m_backward = false ; }

	// ulozeni stareho
	m_old.m_position = m_position;
	
	// pootoceni
	m_rotation.x += m_rotX;
	m_rotation.y += m_rotY;
	m_rotation.z += m_rotZ;
	
	// reset pohybu
	m_req_movement.x = 0;
	m_req_movement.y = 0;
	m_req_movement.z = 0;

	// objekt je rozen jen pokud je aktivni - jinak ale muze klouzat, takze se to nesmi hned vsechno vyhodit
	if (m_active) {
	
		// nastaveni req_move
		if (m_up)	 	 m_req_movement.y =  m_speed;
		if (m_down)     m_req_movement.y = -m_speed;
	
	
		float alpha = ((double)m_rotation.y / 360.0) *  (2*PI);
		m_sin_alpha = sin(alpha);
		m_cos_alpha = cos(alpha);
	
		// dopredu dozadu
		if (m_forward)  {
			m_req_movement.x +=   m_sin_alpha*m_speed;
			m_req_movement.z +=  -m_cos_alpha*m_speed;
		}
		if (m_backward) {
			m_req_movement.x +=  -m_sin_alpha*m_speed;
			m_req_movement.z +=   m_cos_alpha*m_speed;
		}

		// strafovani
		if (m_left)  {
			m_req_movement.z +=   -m_sin_alpha*m_speed;
			m_req_movement.x +=   -m_cos_alpha*m_speed;
		}
		if (m_right) {
			m_req_movement.z +=   m_sin_alpha*m_speed;
			m_req_movement.x +=   m_cos_alpha*m_speed;
		}
	}
		
	// zmena nastaveni aktualniho smeru/rychlosti
	m_moveing.x = m_moveing.x * (1 - m_acceleration) + m_acceleration * m_req_movement.x;
	m_moveing.y = m_moveing.y * (1 - m_acceleration) + m_acceleration * m_req_movement.y;
	m_moveing.z = m_moveing.z * (1 - m_acceleration) + m_acceleration * m_req_movement.z;

	// zmena slidovani - snizeni
	m_sliding.x *= m_antislide;
	m_sliding.y *= m_antislide;
	m_sliding.z *= m_antislide;
	
	// vlastni posun souradnic
	m_position.z +=  m_moveing.z + m_sliding.z;
	m_position.x +=  m_moveing.x + m_sliding.x;
	m_position.y += m_moveing.y + m_sliding.y;	
	
	// spocitame stred modelu
	m_model_position.z = m_position.z ; //+ m_3Dmodel->m_model.m_modelCenter.z;
	m_model_position.x = m_position.x ; // + m_3Dmodel->m_model.m_modelCenter.x;
	m_model_position.y = m_position.y + m_3Dmodel->m_modelCenter.y;
	
	// otestujeme na narazy
	m_engine->HitTest(this);
}

/////////////////////////////////////////////////////////////////////////////////////////

void GActiveObject::Render()
{
	// nastavime pozici matic atd.
	if (!RenderRotoMovePushMatrix()) return;

	// kreslime model
	m_3Dmodel->Render(m_position.x, m_position.y, m_position.z);
	
	// obnoveni stare matice - PushMatrix volal RenderRotoMovePushMatrix
	glPopMatrix();
}

/////////////////////////////////////////////////////////////////////////////////////////

bool GActiveObject::RenderRotoMovePushMatrix()
{
	// zkusime umistit objekt do frustumu. Pokud v nem nei, rovnou koncime
	// TODO: Mozna by se melo testovat i s urcitym okolim a ne jen stredni bod.
#ifdef _HAVE_FRUSTUM_
	if (!frustum.PointInFrustum(m_position.x, m_position.y, m_position.z)) return false;
#endif
	
	// Zatek prostorov matice
	glPushMatrix();

	// nastavime pozici modelu

	// posunuti
	glTranslated(m_position.x, m_position.y, m_position.z); 
	
	/* Pripadny ctverec na kolize, aby bylo lepe videt
	glColor3ub( 255, rand()%256, rand()%256);
	glBegin(GL_LINES);
		glVertex3f(- m_size.x, m_size.y, -m_size.z);
		glVertex3f(- m_size.x, m_size.y, +m_size.z);
		glVertex3f(+ m_size.x, m_size.y, -m_size.z);
		glVertex3f(+ m_size.x, m_size.y, +m_size.z);
		glVertex3f(- m_size.x, m_size.y, +m_size.z);
		glVertex3f(+ m_size.x, m_size.y, +m_size.z);
		glVertex3f(- m_size.x, m_size.y, -m_size.z);
		glVertex3f(+ m_size.x, m_size.y, -m_size.z);
	glEnd();
	*/
	
	// rotace 
	glRotated(m_rotation.x, -1.0, 0.0, 0.0); 
	glRotated(m_rotation.y,  0.0, -1.0, 0.0); 
	glRotated(m_rotation.z,  0.0,  0.0, -1.0); 
	
	return true;
}


/////////////////////////////////////////////////////////////////////////////////////////

void GActiveObject::SetAI(GAI *ai)
{
	if (m_ai != NULL) delete m_ai; // kdyby nahodou
	m_ai = ai;
}

/////////////////////////////////////////////////////////////////////////////////////////

bool GActiveObject::Hitted(GActiveObject * /*object*/)
{
	return false; // defaultove se nedomnivame, ze je treba resit kolizi objektu
}
