//UTILS

// HTextureStore.cpp: implementation of the HTextureStore class.
//
//////////////////////////////////////////////////////////////////////

#include "ph.h"
#include "HTextureStore.h"

HTextureStore htexture;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

HTextureStore::HTextureStore()
{
	// nulovani dat
	m_textures = 0;
	for (int i = 0; i < MAX_TEXTURES; i++) m_texture[i] = NULL;

	// defaultove parametry
	m_mipmaps    = false;
	m_min_filter = GL_LINEAR;
	m_mag_filter = GL_LINEAR;
}

HTextureStore::~HTextureStore()
{
	// smazani seznamu textur
	DestroyAllTextures();
}


// nacteni textury s pouzitim standartnich filtru
int HTextureStore::GetTexture(const char *file)
{
	return GetTexture(file, m_min_filter, m_mag_filter, m_mipmaps);
}

int HTextureStore::GetTexture(const char* file, int min_filter, int mag_filter, bool mipmaps)
{
	
	// prevedeni na mala pismenka
	char file2[150];
	int i;
	for (i = 0; file[i] != '\0'; i++) file2[i] = (file[i] >= 'A' && file[i] <= 'Z')? (file[i] - 'A' + 'a') : file[i];
	file2[i] = '\0';
	
	// hledani jiz jednou vytvorene textury
	for (i = 0; i < MAX_TEXTURES; i++)
		if (m_texture[i] != NULL && strcmp(file2, m_texture[i]->file) == 0) break;
			
	// texturu uz nekdo chtel - nebude placat pameti
	if (i < MAX_TEXTURES) {
		m_texture[i]->used++;
		return i;
	}

	// v pripade neuspechu vytvoreni nove
	return CreateTexture(file2, min_filter, mag_filter, mipmaps);
}

int HTextureStore::CreateTexture(const char* file, int min_filter, int mag_filter, bool mipmaps)
{
	// test na volne misto
	if (m_textures >= MAX_TEXTURES) {
		fprintf(stderr, "file: %s, line: %d MAX_TEXTURES too small - please use larger number.\n", __FILE__, __LINE__);
		return -1;
	}

	// nacteni obrazku
	SDL_Surface *surface;
	surface = LoadBitmap(file, false);
	if (surface == NULL) return -1;

	// vytvoreni objektu textury
	st_texture *t = new st_texture;
	if (t == NULL) return -1;

	// vygenerovani textury
	glGenTextures(1, &t->gl_texture);

	// nastaveni textury
	glBindTexture(GL_TEXTURE_2D, t->gl_texture);

	// predani parametru
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, mag_filter);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, min_filter);

	// pridani barevne mapy (obrazku) - pripadne generovani mip map
	if (mipmaps) 
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, surface->w, surface->h, GL_RGB, GL_UNSIGNED_BYTE, surface->pixels);
	else
		glTexImage2D(GL_TEXTURE_2D, 0, 3, surface->w, surface->h, 0, GL_RGB, GL_UNSIGNED_BYTE, surface->pixels);

	// pridani textury do seznamu - nalezeni prvniho prazdneho mista
	int i;
	for (i = 0; i < MAX_TEXTURES; i++) 
		if (m_texture[i] == NULL) break;

	// okopirujeme data
	strcpy(t->file, file);
	t->used = 1;
	t->alpha = false;
	t->blending = false;
	m_texture[i] = t;
	m_textures++;

	// uklid - smazani SDL_Surface
	SDL_FreeSurface(surface);
	surface = NULL;
//	delete surface; - to ty byt nesmi - zere to pamet

	// konec 
	return i;
}

// nastavi texturu do OpenGL
void HTextureStore::SetTexture(int ID)
{
	// musi se nastavit i barva - mixuje se to sni
	if (ID == -1) return;

	if (m_texture[ID] != NULL) {
 		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, m_texture[ID]->gl_texture);
		
		// tady tohle by tu casem mozna melo byt
		if (m_texture[ID]->blending) {
			glEnable(GL_BLEND);
			glEnable(GL_ALPHA_TEST);
			glColor4f(1.0, 1.0, 1.0, 1.0);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); // GL_ONE_MINUS_SRC_ALPHA
			glAlphaFunc(GL_ALWAYS, 0.0);
		} else 
		if (m_texture[ID]->alpha) {
			glEnable(GL_BLEND);
			glEnable(GL_ALPHA_TEST);
			glColor4f(1.0, 1.0, 1.0, 1.0);
		//	glDisable(GL_DEPTH_TEST);                                                       // Disables Depth Testing
//		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); // GL_ONE_MINUS_SRC_ALPHA
		glBlendFunc(GL_ONE, GL_ZERO); // GL_ONE_MINUS_SRC_ALPHA
		//	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		//	glEnable(GL_ALPHA_TEST);
		glAlphaFunc(GL_GREATER, 0.2);
		//	glEnable(GL_BLEND);
		}
		else {
			glColor3ub(255, 255, 255);           // Nastavime standart barvu
			glEnable(GL_DEPTH_TEST); 
			glDisable(GL_BLEND);
			glDisable(GL_ALPHA_TEST);
		}			
	}
}

// vypne texturovani v OpenGL
void HTextureStore::SetNullTexture()
{
	glBindTexture(GL_TEXTURE_2D, 0);  // nastaveni bez textury
	glDisable(GL_TEXTURE_2D);			    // zakaz texturovani
	glColor3ub(255, 255, 255);           // Reset the color to normal again
}

/////////////////////////////////////////////////////////////////////////////////////////

SDL_Surface* HTextureStore::LoadBitmap(const char *filename, bool convert_to_smallchars/* = true*/)
{
   Uint8 *rowhi, *rowlo;
   Uint8 *tmpbuf, tmpch;
   SDL_Surface *image;
   int i, j;


	// prevedeni na mala pismenka - tohle je dost debilni - chtelo by to neco, co by uzivalo standartni knihovni fce
	char file2[MAX_PATH];
	if (convert_to_smallchars) {
		int i;
		for (i = 0; filename[i] != '\0'; i++) file2[i] = (filename[i] >= 'A' && filename[i] <= 'Z')? (filename[i] - 'A' + 'a') : filename[i];
		file2[i] = '\0';
	}
	else strcpy(file2, filename);

	// pokusime se texturu najit Kdyz se to nepovede, find_file na stderr vypisuje hlasku
	if (!find_file(file2)) return NULL;
	
	// nacteni obrazkovych dat
	image = IMG_Load(file2);
   if ( image == NULL ) {
		fprintf(stderr, gettext("Unable to load %s: %s\n"), file2, SDL_GetError());
      return(NULL);
   }

    // GL surfaces jsou vzhuru nohama
   tmpbuf = (Uint8 *)malloc(image->pitch);

   if ( tmpbuf == NULL ) {
       fprintf(stderr, "Out of memory\n");
       return NULL;
   }

   rowhi = (Uint8 *)image->pixels;
   rowlo = rowhi + (image->h * image->pitch) - image->pitch;
   for ( i=0; i<image->h/2; ++i ) {
		// prevraceni BGR na RGB
		if (image->format->Bshift == 0)
			for ( j=0; j<image->w; ++j ) {
            tmpch = rowhi[j*3];
            rowhi[j*3] = rowhi[j*3+2];
            rowhi[j*3+2] = tmpch;
            tmpch = rowlo[j*3];
            rowlo[j*3] = rowlo[j*3+2];
            rowlo[j*3+2] = tmpch;
			}

      memcpy(tmpbuf, rowhi, image->pitch);
      memcpy(rowhi, rowlo, image->pitch);
      memcpy(rowlo, tmpbuf, image->pitch);
      rowhi += image->pitch;
      rowlo -= image->pitch;
   }

	// uklidime
   free(tmpbuf);

   return image;
}

/////////////////////////////////////////////////////////////////////////////////////////

void HTextureStore::RemoveTexture(int ID)
{
	if (ID == -1 || m_texture[ID] == NULL) return;

	// smazeme jedno pouziti a zkusime, zda na texturu jeste nekdo odkazuje 
 	if (--m_texture[ID]->used <= 0) {
      // fyzicke zruseni textury
		DestroyTexture(ID);
	}
}

void HTextureStore::DestroyAllTextures()
{
	for (int i = 0; i < MAX_TEXTURES; i++) DestroyTexture(i);
}

void HTextureStore::DestroyTexture(int ID)
{
	if (m_texture[ID] == NULL) return;

 	glDeleteTextures(1, &m_texture[ID]->gl_texture);
	
	delete m_texture[ID];
	m_texture[ID] = NULL;
	
	m_textures--;
}

// nastavi qvalitu nacitanych textur
void HTextureStore::SetTextureGeneratingQuality(int quality)
{
	switch (quality) {
	case 0 :
		m_mipmaps    = false;
		m_min_filter = GL_NEAREST;
		m_mag_filter = GL_NEAREST;
		break;
	case 1 :
		m_mipmaps    = false;
		m_min_filter = GL_LINEAR;
		m_mag_filter = GL_LINEAR;
		break;
	case 2 :
		m_mipmaps    = true;
		m_min_filter = GL_LINEAR_MIPMAP_NEAREST;
		m_mag_filter = GL_LINEAR;
		break;
	}
}

// nacita texturu vcetne alpha hodnoty - zatim nedodelane!!!
int HTextureStore::GetTextureWithAlpha(const char *file, const char *alphafile)
{
	
	char file2[MAX_PATH];
	strcpy(file2, file);
	
	// pokusime se texturu najit Kdyz se to nepovede, find_file na stderr vypisuje hlasku
	if (!find_file(file2)) return -1;
	
	// nacteni obrazku
	SDL_Surface *surface;
	surface = IMG_Load(file2);
	if (surface == NULL) return -1;

	// pokusime se texturu najit Kdyz se to nepovede, find_file na stderr vypisuje hlasku
	strcpy(file2, alphafile);
	if (!find_file(file2)) return -1;
		
	
	// nacteni alphy obrazku
	SDL_Surface *alpha;
	alpha = IMG_Load(file2);
	if (alpha == NULL) return -1;
	
	Uint8 *rowtmp, *rowalpha, *rowimg;
   Uint8 *tmpbuf;
   int i, j;
	
    // GL surfaces jsou vzhuru nohama
   tmpbuf = (Uint8 *)malloc(((surface->pitch * surface->h)*4)/3 );

   if ( tmpbuf == NULL ) {
       fprintf(stderr, gettext("Out of memory while loading texture %s\n"), file2);
       return -1;
   }

   rowimg = (Uint8 *)surface->pixels;
   rowalpha = (Uint8 *)alpha->pixels;
   rowtmp = tmpbuf + ((surface->h * surface->pitch - surface->pitch)*4)/3;
   for ( i = 0; i < surface->h; i++ ) {
		// prevraceni BGR na RGB
		if (surface->format->Bshift == 0)
			for ( j = 0; j < surface->w; j++ ) {
            rowtmp[j*4  ] = rowimg[j*3+2];
            rowtmp[j*4+1] = rowimg[j*3+1];
            rowtmp[j*4+2] = rowimg[j*3  ];
            rowtmp[j*4+3] = rowalpha[j*3];
			}
		else 
 			for ( j = 0; j < surface->w; j++ ) {
            rowtmp[j*4 ]  = rowimg[j*3  ];
            rowtmp[j*4+1] = rowimg[j*3+1];
            rowtmp[j*4+2] = rowimg[j*3+2];
            rowtmp[j*4+3] = rowalpha[j*3];
			}
      rowimg += surface->pitch;
      rowalpha += surface->pitch;
      rowtmp -= (surface->pitch*4)/3;
   }

	
	// vytvoreni objektu textury
	st_texture *t = new st_texture;
	if (t == NULL) return -1;

	// vygenerovani textury
	glGenTextures(1, &t->gl_texture);

	// nastaveni textury
	glBindTexture(GL_TEXTURE_2D, t->gl_texture);

	// predani parametru
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, m_mag_filter);
	 // pri alphe nemuze byt mni filtr jako mimmapy :-(
	if (m_min_filter == GL_LINEAR_MIPMAP_NEAREST) glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	else glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, m_min_filter);
		

	// pridani barevne mapy (obrazku) - pripadne generovani mip map
//	if (m_mipmaps)  ---- mipmapy u s alphou byt nemohou - mrvi se to.
//		gluBuild2DMipmaps(GL_TEXTURE_2D, 4, surface->w, surface->h, GL_RGBA, GL_UNSIGNED_BYTE, tmpbuf);
//	else 
		glTexImage2D(GL_TEXTURE_2D, 0, 4, surface->w, surface->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tmpbuf);

	// pridani textury do seznamu - nalezeni prvniho prazdneho mista
//	int i;
	for (i = 0; i < MAX_TEXTURES; i++) 
		if (m_texture[i] == NULL) break;

	// okopirujeme data
	strcpy(t->file, file);
	t->used = 1;
	t->alpha = true;
	t->blending = false;
	m_texture[i] = t;
	m_textures++;

	// uklid - smazani SDL_Surface
	SDL_FreeSurface(surface);
	surface = NULL;
	SDL_FreeSurface(alpha);
	alpha = NULL;
	//	delete surface; - to ty byt nesmi - zere to pamet

	// uklidime
   free(tmpbuf);	
		
	// konec 
	return i;	
}

// pokusi se k souboru najit i soubor s maskou - je stejny jako puvodni, ale na konci ma navic misto *.jpg *+.jpg   pro pripad, ze to byl jpeg...
int HTextureStore::GetTextureAutoAlpha(const char *file)
{
	// prevedeni na mala pismenka
	char file2[150];
	char file3[150];
	
	int i;
	for (i = 0; file[i] != '\0'; i++) {
		file2[i] = (file[i] >= 'A' && file[i] <= 'Z')? (file[i] - 'A' + 'a') : file[i];
		file3[i] = file2[i];
	}
	
	// tohle je velice provizorni - proste prida + pred konec sourobu - pipony mohou byt jen 3 znaky dlouhe
	file2[i] = '\0';
	file3[i-4] = '+';
	file3[i-3] = file2[i-4];
	file3[i-2] = file2[i-3];
	file3[i-1] = file2[i-2];
	file3[i]   = file2[i-1];
	file3[i+1] = '\0';
	
	
	// pokusime se najit soubor s alphou, ale nevypisujeme, kdyz chybi..
	if (find_file(file3, false))
		return GetTextureWithAlpha(file2, file3);
	else
		return GetTexture(file2);
}
	
// nacteni probehne stejne, jako u alphy, ale pridame, ze chceme pisto pouhe alphy i blending
int HTextureStore::GetTextureWithBlend(const char *file, const char *blendfile)
{
	int i = GetTextureWithAlpha(file, blendfile);
	if ( i == -1 ) return -1;
	
	m_texture[i]->blending = true;
	return i;
}

	// pokusi se k souboru najit i soubor s maskou - je stejny jako puvodni, ale na konci ma navic misto *.jpg *+.jpg   pro pripad, ze to byl jpeg...
int HTextureStore::GetTextureAutoBlend(const char *file)
{
	// prevedeni na mala pismenka
	char file2[150];
	char file3[150];
	
	int i;
	for (i = 0; file[i] != '\0'; i++) {
		file2[i] = (file[i] >= 'A' && file[i] <= 'Z')? (file[i] - 'A' + 'a') : file[i];
		file3[i] = file2[i];
	}
	
	// tohle je velice provizorni - proste prida - pred konec sourobu - pipony mohou byt jen 3 znaky dlouhe
	file2[i] = '\0';
	file3[i-4] = '-';
	file3[i-3] = file2[i-4];
	file3[i-2] = file2[i-3];
	file3[i-1] = file2[i-2];
	file3[i]   = file2[i-1];
	file3[i+1] = '\0';
	
	
	// pokusime se najit soubor s blendem, ale nevypisujeme, kdyz chybi..
	if (find_file(file3, false))
		return GetTextureWithBlend(file2, file3);
	else
		return GetTexture(file2);	
}

// i s Alpha/Blend - podle toho, co pujde
int HTextureStore::GetTextureAuto(const char *file)
{
	// prevedeni na mala pismenka
	char file2[150];
	char file3[150];
	
	int i;
	for (i = 0; file[i] != '\0'; i++) {
		file2[i] = (file[i] >= 'A' && file[i] <= 'Z')? (file[i] - 'A' + 'a') : file[i];
		file3[i] = file2[i];
	}
	
	file2[i] = '\0';
	file3[i-4] = '-';
	file3[i-3] = file2[i-4];
	file3[i-2] = file2[i-3];
	file3[i-1] = file2[i-2];
	file3[i]   = file2[i-1];
	file3[i+1] = '\0';

	// nejdrive se pokusime o blending, jinak o alphu a nakonec obyc :-(	
	if (find_file(file3, false))
		return GetTextureWithBlend(file2, file3);
	else {
		file3[i-4] = '+'; // zmenime na alpha soubor
		if (find_file(file3, false))
			return GetTextureWithAlpha(file2, file3);
		else
			return GetTexture(file2);
	}			
}
